@@ -3,7 +3,20 @@
 {-# LANGUAGE DeriveFoldable #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
--- | This is the main entry point for using http-client. Used by itself, this
+-- |
+--
+-- = Simpler API
+--
+-- The API below is rather low-level. The @Network.HTTP.Simple@ module (from
+-- the @http-conduit@ package) provides a higher-level API with built-in
+-- support for things like JSON request and response bodies. For most users,
+-- this will be an easier place to start. You can read the tutorial at:
+--
+-- https://github.com/commercialhaskell/jump/blob/master/doc/http-client.md
+--
+-- = Lower-level API
+--
+-- This is the main entry point for using http-client. Used by itself, this
 -- module provides low-level access for streaming request and response bodies,
 -- and only non-secure HTTP connections. Helper packages such as http-conduit
 -- provided higher level streaming approaches, while other helper packages like
@@ -215,19 +228,19 @@ responseOpenHistory req0 man = do
     reqRef <- newIORef req0
     historyRef <- newIORef id
     let go req = do
-            res <- httpRaw req man
+            (req', res) <- httpRaw' req man
             case getRedirectedRequest
-                    req
+                    req'
                     (responseHeaders res)
                     (responseCookieJar res)
                     (statusCode $ responseStatus res) of
-                Nothing -> return (res, Nothing)
-                Just req' -> do
-                    writeIORef reqRef req'
+                Nothing -> return (res, req', False)
+                Just req'' -> do
+                    writeIORef reqRef req''
                     body <- brReadSome (responseBody res) 1024
                     modifyIORef historyRef (. ((req, res { responseBody = body }):))
-                    return (res, Just req')
-    res <- httpRedirect (redirectCount req0) go req0
+                    return (res, req'', True)
+    (_, res) <- httpRedirect' (redirectCount req0) go req0
     reqFinal <- readIORef reqRef
     history <- readIORef historyRef
     return HistoriedResponse
